<xsl:stylesheet version = "1.0"
		xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
		xmlns:a="http://alvis.info/enriched/"
		>

  <xsl:output method="xml" indent="yes" doctype-system="full-la-tag.dtd" />
  
  <xsl:key name="ne-key" match="a:named_entity" use="a:list_refid_token/a:refid_token" />
  <xsl:key name="word-key" match="a:word" use="a:list_refid_token/a:refid_token" />
  <xsl:key name="lemma-key" match="a:lemma" use="a:refid_word" />
  <xsl:key name="pos-key" match="a:morphosyntactic_features" use="a:refid_word" />
  <xsl:key name="end-sentence-key" match="a:sentence" use="a:refid_end_token" />


  <xsl:template match="a:documentCollection">
    <xsl:apply-templates select="./a:documentRecord/a:linguisticAnalysis" />
  </xsl:template>

  <xsl:template match="a:linguisticAnalysis">
    <xsl:element name="document">
      <xsl:element name="id">
	<xsl:value-of select="../@id" />
      </xsl:element>
      <xsl:apply-templates select="./a:sentence_level/a:sentence" />
    </xsl:element>
  </xsl:template>
  
  <xsl:template match="a:sentence">
    <xsl:variable name="tok-start" select="./a:refid_start_token" />
    <xsl:variable name="tok-end" select="./a:refid_end_token" />
    <xsl:variable name="sosent" select="../../a:token_level/a:token[a:id = $tok-start]/a:from" />
    <xsl:variable name="eosent" select="../../a:token_level/a:token[a:id = $tok-end]/a:to" />
    <xsl:element name="text">
      <xsl:apply-templates select="../../a:token_level/a:token[a:from >= $sosent and $eosent >= a:to]">
	<xsl:sort select="a:from" data-type="number" />
      </xsl:apply-templates>
    </xsl:element>
  </xsl:template>

  <xsl:template match="a:token">
    <xsl:variable name="tokid" select="a:id" />
    <xsl:variable name="ne" select="key('ne-key',$tokid)" />
    <xsl:variable name="word" select="key('word-key',$tokid)" />
    <xsl:choose>
      <xsl:when test="$ne">
	<xsl:if test="$ne/a:list_refid_token/a:refid_token[1]=$tokid">
	  <xsl:element name="ne">
	    <xsl:attribute name="canonical">
	      <xsl:value-of select="$ne/a:canonical_form" />
	    </xsl:attribute>
	    <xsl:attribute name="type">
	      <xsl:value-of select="$ne/a:named_entity_type" />
	    </xsl:attribute>
	    <xsl:attribute name="alvis-id">
	      <xsl:value-of select="$ne/a:id" />
	    </xsl:attribute>
	    <xsl:value-of select="$ne/a:form" />
	  </xsl:element>
	</xsl:if>
      </xsl:when>
      <xsl:when test="$word">
	<xsl:if test="$word/a:list_refid_token/a:refid_token[1]=$tokid">
	  <xsl:variable name="lemma" select="key('lemma-key',$word/a:id)" />
	  <xsl:variable name="pos" select="key('pos-key',$word/a:id)" />
	  <xsl:element name="word">
	    <xsl:attribute name="alvis-id">
	      <xsl:value-of select="$word/a:id" />
	    </xsl:attribute>
	    <xsl:attribute name="lemma">
	      <xsl:value-of select="$lemma/a:canonical_form" />
	    </xsl:attribute>
	    <xsl:attribute name="penn-tag">
	      <xsl:value-of select="$pos/a:syntactic_category" />
	    </xsl:attribute>
	    <xsl:choose>
	      <xsl:when test="starts-with($pos/a:syntactic_category,'J')">
		<xsl:attribute name="category">J</xsl:attribute>
	      </xsl:when>
	      <xsl:when test="starts-with($pos/a:syntactic_category,'N')">
		<xsl:attribute name="category">N</xsl:attribute>
	      </xsl:when>
	      <xsl:when test="starts-with($pos/a:syntactic_category,'PP')">
		<xsl:attribute name="category">PP</xsl:attribute>
	      </xsl:when>
	      <xsl:when test="starts-with($pos/a:syntactic_category,'RB')">
		<xsl:attribute name="category">RB</xsl:attribute>
	      </xsl:when>
	      <xsl:when test="starts-with($pos/a:syntactic_category,'V')">
		<xsl:attribute name="category">V</xsl:attribute>
	      </xsl:when>
	      <xsl:otherwise>
		<xsl:attribute name="category">O</xsl:attribute>
	      </xsl:otherwise>
	    </xsl:choose>
	    <xsl:value-of select="$word/a:form" />
	  </xsl:element>
	</xsl:if>
      </xsl:when>
      <xsl:otherwise>
	<xsl:value-of select="a:content" />
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

</xsl:stylesheet>
