#include "attributes.h"
#include "wchar_utils.h"

#include <stdlib.h>
#include <bunch.h>

#include "mouchard.h"

#ifdef TRISH2
patricia_t *alist_new(int size) {
  return patricia_new(DEFAULT_ALIST_NB, DEFAULT_ALIST_SB, bunch_new(sizeof(attribute_t), DEFAULT_ALIST_SIZE));
}

void alist_free(patricia_t *alist) {
  unsigned int i, j;

  for (i = 0; i < alist->data->block_number; i++)
    for (j = 0; j < alist->data->block_size - alist->data->blocks[i]->unclaimed; j++) {
      attribute_t *attr = ((attribute_t *)alist->data->blocks[i]->data) + j;
      if (attr->attr_type == ATTRIBUTE_TYPE_RE) {
	regfree(((attr_re_t *)attr->attr_obj)->are_cre);
	free(((attr_re_t *)attr->attr_obj)->are_cre);	
      }
      free(attr->attr_obj);
    }
  patricia_free(alist);
}

int alist_add_re(patricia_t *alist, const char *name, char *re) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_RE;
  attr->attr_obj = malloc(sizeof(attr_re_t));
  ((attr_re_t *)attr->attr_obj)->are_re = re;
  ((attr_re_t *)attr->attr_obj)->are_cre = malloc(sizeof(regex_t));
  regcomp(((attr_re_t *)attr->attr_obj)->are_cre, re, REG_EXTENDED | REG_NOSUB);

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_dict(patricia_t *alist, const char *name, char *dict, int op) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_DICT;
  attr->attr_obj = malloc(sizeof(attr_dict_t));
  ((attr_dict_t *)attr->attr_obj)->adict_name = dict;
  ((attr_dict_t *)attr->attr_obj)->adict_op = op;

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_length(patricia_t *alist, const char *name) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_LENGTH;
  attr->attr_obj = NULL;

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_signal(patricia_t *alist, const char *name, const char *word, int range, int dir) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_SIGNAL;
  attr->attr_obj = malloc(sizeof(attr_signal_t));
  ((attr_signal_t *)attr->attr_obj)->asig_word = word;
  ((attr_signal_t *)attr->attr_obj)->asig_range = range;
  ((attr_signal_t *)attr->attr_obj)->asig_direction = dir;

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_pos(patricia_t *alist, const char *name, int range, int dir) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_POS;
  attr->attr_obj = malloc(sizeof(attr_pos_t));
  ((attr_pos_t *)attr->attr_obj)->apos_range = range;
  ((attr_pos_t *)attr->attr_obj)->apos_direction = dir;

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_context(patricia_t *alist, const char *name) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_CONTEXT;
  attr->attr_obj = NULL;

  pmap_add(alist, w, attr, 0);

  return 1;
}

int alist_add_label(patricia_t *alist, const char *name) {
  attribute_t *attr = bunch_claim(alist->data, 1);
  wchar_t *w = char2wcs(name);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_LABEL;
  attr->attr_obj = NULL;

  pmap_add(alist, w, attr, 0);

  return 1;
}





void attributes_print(FILE *f, patricia_t *al) {
  unsigned int i, j;

  for (i = 0; i < al->data->block_number; i++)
    for (j = 0; j < al->data->block_size - al->data->blocks[i]->unclaimed; j++) {
      attribute_t *attr = ((attribute_t *)al->data->blocks[i]->data) + j;
      
      fprintf(f, "% 3d %s: ", i, attr->attr_name);
      switch (attr->attr_type) {
      case ATTRIBUTE_TYPE_RE:
	fprintf(f, "RE (%s, %p)\n", ((attr_re_t *)attr->attr_obj)->are_re, (void *)((attr_re_t *)attr->attr_obj)->are_cre);
	break;
      case ATTRIBUTE_TYPE_DICT:
	fprintf(f, "DICT (%s, %d)\n", ((attr_dict_t *)attr->attr_obj)->adict_name, ((attr_dict_t *)attr->attr_obj)->adict_op);
	break;
      case ATTRIBUTE_TYPE_LENGTH:
	fprintf(f, "LENGTH\n");
	break;
      case ATTRIBUTE_TYPE_SIGNAL:
	fprintf(f, "SIGNAL (%s, %d, %d)\n", ((attr_signal_t *)attr->attr_obj)->asig_word, ((attr_signal_t *)attr->attr_obj)->asig_range, ((attr_signal_t *)attr->attr_obj)->asig_direction);
	break;
      case ATTRIBUTE_TYPE_POS:
	fprintf(f, "SIGNAL (%d, %d)\n", ((attr_pos_t *)attr->attr_obj)->apos_range, ((attr_pos_t *)attr->attr_obj)->apos_direction);
	break;
      case ATTRIBUTE_TYPE_CONTEXT:
	fprintf(f, "CONTEXT\n");
	break;
      case ATTRIBUTE_TYPE_LABEL:
	fprintf(f, "LABEL\n");
	break;
      }
    }
}

#else

association_t *alist_new(int size) {
  return association_new(sizeof(attribute_t), size);
}

void alist_free(association_t *alist) {
  bunch_element_ref_t i;

  for (i = 1; i < alist->values->next_ref; i++) {
    attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, i);
    if (attr->attr_type == ATTRIBUTE_TYPE_RE) {
      regfree(((attr_re_t *)attr->attr_obj)->are_cre);
      free(((attr_re_t *)attr->attr_obj)->are_cre);
    }
    free(attr->attr_obj);
  }
  association_free(alist);
}

int alist_add_re(association_t *alist, const char *name, char *re) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_RE;
  attr->attr_obj = malloc(sizeof(attr_re_t));
  ((attr_re_t *)attr->attr_obj)->are_re = re;
  ((attr_re_t *)attr->attr_obj)->are_cre = malloc(sizeof(regex_t));
  regcomp(((attr_re_t *)attr->attr_obj)->are_cre, re, REG_EXTENDED | REG_NOSUB);

  return 1;
}

int alist_add_dict(association_t *alist, const char *name, char *dict, int op) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_DICT;
  attr->attr_obj = malloc(sizeof(attr_dict_t));
  ((attr_dict_t *)attr->attr_obj)->adict_name = dict;
  ((attr_dict_t *)attr->attr_obj)->adict_op = op;

  return 1;
}

int alist_add_length(association_t *alist, const char *name) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_LENGTH;
  attr->attr_obj = NULL;

  return 1;
}

int alist_add_signal(association_t *alist, const char *name, const char *word, int range, int dir) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_SIGNAL;
  attr->attr_obj = malloc(sizeof(attr_signal_t));
  ((attr_signal_t *)attr->attr_obj)->asig_word = word;
  ((attr_signal_t *)attr->attr_obj)->asig_range = range;
  ((attr_signal_t *)attr->attr_obj)->asig_direction = dir;

  return 1;
}

int alist_add_pos(association_t *alist, const char *name, int range, int dir) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_POS;
  attr->attr_obj = malloc(sizeof(attr_pos_t));
  ((attr_pos_t *)attr->attr_obj)->apos_range = range;
  ((attr_pos_t *)attr->attr_obj)->apos_direction = dir;

  return 1;
}

int alist_add_context(association_t *alist, const char *name) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_CONTEXT;
  attr->attr_obj = NULL;

  return 1;
}

int alist_add_label(association_t *alist, const char *name) {
  bunch_element_ref_t ref = association_insert(alist, name, NULL, 1);
  attribute_t *attr = BUNCH_ELT(attribute_t, alist->values, ref);

  if (attr == NULL) {
    return 0;
  }

  attr->attr_name = name;
  attr->attr_type = ATTRIBUTE_TYPE_LABEL;
  attr->attr_obj = NULL;

  return 1;
}





void attributes_print(FILE *f, association_t *al) {
  int i;

  for (i = 1; i < al->values->next_ref; i++) {
    attribute_t *attr = BUNCH_ELT(attribute_t, al->values, i);
    fprintf(f, "% 3d %s: ", i, attr->attr_name);
    switch (attr->attr_type) {
    case ATTRIBUTE_TYPE_RE:
      fprintf(f, "RE (%s, %p)\n", ((attr_re_t *)attr->attr_obj)->are_re, (void *)((attr_re_t *)attr->attr_obj)->are_cre);
      break;
    case ATTRIBUTE_TYPE_DICT:
      fprintf(f, "DICT (%s, %d)\n", ((attr_dict_t *)attr->attr_obj)->adict_name, ((attr_dict_t *)attr->attr_obj)->adict_op);
      break;
    case ATTRIBUTE_TYPE_LENGTH:
      fprintf(f, "LENGTH\n");
      break;
    case ATTRIBUTE_TYPE_SIGNAL:
      fprintf(f, "SIGNAL (%s, %d, %d)\n", ((attr_signal_t *)attr->attr_obj)->asig_word, ((attr_signal_t *)attr->attr_obj)->asig_range, ((attr_signal_t *)attr->attr_obj)->asig_direction);
      break;
    case ATTRIBUTE_TYPE_POS:
      fprintf(f, "SIGNAL (%d, %d)\n", ((attr_pos_t *)attr->attr_obj)->apos_range, ((attr_pos_t *)attr->attr_obj)->apos_direction);
      break;
    case ATTRIBUTE_TYPE_CONTEXT:
      fprintf(f, "CONTEXT\n");
      break;
    case ATTRIBUTE_TYPE_LABEL:
      fprintf(f, "LABEL\n");
      break;
    }
  }
}
#endif
