#ifndef HAVE_ATTRIBUTES_H
#define HAVE_ATTRIBUTES_H

#include "defaults.h"

#include <regex.h>

#ifdef TRISH2
#  include <patricia_map.h>
#else
#  include <association.h>
#endif

#define ATTRIBUTE_TYPE_RE      1
#define ATTRIBUTE_TYPE_DICT    2
#define ATTRIBUTE_TYPE_LENGTH  3
#define ATTRIBUTE_TYPE_SIGNAL  4
#define ATTRIBUTE_TYPE_POS     5
#define ATTRIBUTE_TYPE_CONTEXT 6
#define ATTRIBUTE_TYPE_LABEL   7

typedef struct {
  int attr_type;
  const char *attr_name;
  void *attr_obj;
} attribute_t;

typedef struct {
  char *are_re;
  regex_t *are_cre;
} attr_re_t;

#define ATTR_DICT_OP_EQUALS   1
#define ATTR_DICT_OP_CONTAINS 2

typedef struct {
  char *adict_name;
  int adict_op;
} attr_dict_t;

typedef struct {
  const char *asig_word;
  int asig_range;
  int asig_direction;
} attr_signal_t;

typedef struct {
  int apos_range;
  int apos_direction;
} attr_pos_t;

#ifdef TRISH2
patricia_t *alist_new(int size);
void alist_free(patricia_t *alist);

int alist_add_re(patricia_t *alist, const char *name, char *re);
int alist_add_dict(patricia_t *alist, const char *name, char *dict, int op);
int alist_add_length(patricia_t *alist, const char *name);
int alist_add_signal(patricia_t *alist, const char *name, const char *word, int range, int dir);
int alist_add_pos(patricia_t *alist, const char *name, int range, int dir);
int alist_add_context(patricia_t *alist, const char *name);
int alist_add_label(patricia_t *alist, const char *name);

void attributes_print(FILE *f, patricia_t *al);
#else
association_t *alist_new(int size);
void alist_free(association_t *alist);

int alist_add_re(association_t *alist, const char *name, char *re);
int alist_add_dict(association_t *alist, const char *name, char *dict, int op);
int alist_add_length(association_t *alist, const char *name);
int alist_add_signal(association_t *alist, const char *name, const char *word, int range, int dir);
int alist_add_pos(association_t *alist, const char *name, int range, int dir);
int alist_add_context(association_t *alist, const char *name);
int alist_add_label(association_t *alist, const char *name);

void attributes_print(FILE *f, association_t *al);
#endif

#endif
