/*
  Command-line options configuration for the RenBio executable

*/

#ifndef HAVE_CL_CONFIG_H
#define HAVE_CL_CONFIG_H

/*
  CLO_LONG_*
  Long versions of command-mine options to be used with two dashes
*/

#define CLO_LONG_QUIET       "quiet"
#define CLO_LONG_VERBOSE     "verbose"
#define CLO_LONG_LOCACE      "locace"
#define CLO_LONG_STRICT      "strict"
#define CLO_LONG_FORCE       "force"
#define CLO_LONG_DICT        "dictionary"
#define CLO_LONG_NP_LENGTH   "noun-phrase-length"
#define CLO_LONG_CTX         "context-value"
#define CLO_LONG_LOG         "log"
#define CLO_LONG_TYPE        "type"
#define CLO_LONG_ALVIS       "alvis"
#define CLO_LONG_REANNOT     "reannot"


/*
  CLO_SHORT_*
  Short versions of command-mine options to be used with a single dash
*/

#define CLO_SHORT_QUIET       'q'
#define CLO_SHORT_VERBOSE     'v'
#define CLO_SHORT_LOCACE      'V'
#define CLO_SHORT_STRICT      'W'
#define CLO_SHORT_FORCE       'F'
#define CLO_SHORT_DICT        'd'
#define CLO_SHORT_NP_LENGTH   'l'
#define CLO_SHORT_CTX         'C'
#define CLO_SHORT_LOG         'L'
#define CLO_SHORT_TYPE        't'
#define CLO_SHORT_ALVIS       'A'
#define CLO_SHORT_REANNOT     'r'


/*
  CLO_SWITCH_*
  Return values to use with libpopt
*/

#define CLO_SWITCH_QUIET    1
#define CLO_SWITCH_VERBOSE  2
#define CLO_SWITCH_LOCACE   3
#define CLO_SWITCH_STRICT   4
#define CLO_SWITCH_FORCE    5
#define CLO_SWITCH_DICT     6
#define CLO_SWITCH_CTX      7
#define CLO_SWITCH_TYPE     8


/*
  CLO_HELP_*
  Common help texts
*/

#define CLO_HELP_QUIET         "Be quiet"
#define CLO_HELP_VERBOSE       "Be verbose"
#define CLO_HELP_LOCACE        "Be even more verbose"
#define CLO_HELP_STRICT        "Abort when a warning is issued"
#define CLO_HELP_FORCE         "Proceed even after errors (may cause instability)"
#define CLO_HELP_DICT          "Name and path to a dictionary"
#define CLO_HELP_NP_LENGTH     "Maximum length of noun phrases"
#define CLO_HELP_CTX           "Value of the context attribute"
#define CLO_HELP_LOG           "File where to store the whole path in the decision tree"
#define CLO_HELP_TYPE          "Named entity type to consider"
#define CLO_HELP_ALVIS         "Input in Alvis Enriched Document format"
#define CLO_HELP_REANNOT       "Output in a format suitable for Alvis reannotation"


/*
  Verbose options common to all executables
*/

struct poptOption verb_opts[] = {
  { CLO_LONG_QUIET   , CLO_SHORT_QUIET   , POPT_ARG_NONE  , NULL , CLO_SWITCH_QUIET   , CLO_HELP_QUIET   , NULL },
  { CLO_LONG_VERBOSE , CLO_SHORT_VERBOSE , POPT_ARG_NONE  , NULL , CLO_SWITCH_VERBOSE , CLO_HELP_VERBOSE , NULL },
  { CLO_LONG_LOCACE  , CLO_SHORT_LOCACE  , POPT_ARG_NONE  , NULL , CLO_SWITCH_LOCACE  , CLO_HELP_LOCACE  , NULL },
  { CLO_LONG_STRICT  , CLO_SHORT_STRICT  , POPT_ARG_NONE  , NULL , CLO_SWITCH_STRICT  , CLO_HELP_STRICT  , NULL },
  { CLO_LONG_FORCE   , CLO_SHORT_FORCE   , POPT_ARG_NONE  , NULL , CLO_SWITCH_FORCE   , CLO_HELP_FORCE   , NULL },
  POPT_TABLEEND
};

#endif
