#include "decision_tree.h"
#include "wchar_utils.h"

#include <stdlib.h>

#include "mouchard.h"

dtree_t *dtree_new(int op, int ival, char *sval) {
  dtree_t *dt = malloc(sizeof(dtree_t));

  dt->dtree_op = op;
  dt->dtree_ival = ival;
  dt->dtree_sval = sval;
  dt->dtree_else = NULL;
#ifdef TRISH2
  dt->dtree_attribute = NULL;
#else
  dt->dtree_attribute = 0;
#endif
  dt->dtree_test = NULL;

  return dt;
}

void dtree_free(dtree_t *dt) {
  if (dt == NULL)
    return;
  dtree_free(dt->dtree_else);
  dtree_free(dt->dtree_test);
  free(dt);
}

#ifdef TRISH2
void dtree_attribute(dtree_t *dtree, patricia_t *al, const char *aname) {
  wchar_t *w = char2wcs(aname);
  dtree->dtree_attribute = pmap_get(al, w);
  free(w);
}
#else
void dtree_attribute(dtree_t *dtree, association_t *al, const char *aname) {
  bunch_element_ref_t ref = association_get(al, aname);

  dtree->dtree_attribute = ref;
}
#endif

void dtree_else(dtree_t *dtree, dtree_t *dte) {
  if (dtree->dtree_test == NULL) {
    dtree->dtree_test = dte;
  }
  else {
    dtree = dtree->dtree_test;
    while (dtree->dtree_else != NULL) {
      dtree = dtree->dtree_else;
    }
    dtree->dtree_else = dte;
  }
}

#define DEPTH(d) for (i = d; i; i--) fprintf(f, "  ");

#ifdef TRISH2
void dtree_print(FILE *f, patricia_t *al, dtree_t *dtree, int depth) {
  int i;
  dtree_t *cdt = dtree->dtree_test;

  fprintf(f, "%s\n", dtree->dtree_attribute->attr_name);
  while (cdt) {
    DEPTH(depth);
    fprintf(f, "%s %d/%s ", (cdt->dtree_op == DTREE_OP_EQ ? " =" : (cdt->dtree_op == DTREE_OP_LE ? "<=" : " >")), cdt->dtree_ival, cdt->dtree_sval);
    dtree_print(f, al, cdt, depth + 1);
    cdt = cdt->dtree_else;
  }
}
#else
void dtree_print(FILE *f, association_t *al, dtree_t *dtree, int depth) {
  int i;
  dtree_t *cdt = dtree->dtree_test;

  fprintf(f, "%s\n", BUNCH_ELT(attribute_t, al->values, dtree->dtree_attribute)->attr_name);
  while (cdt) {
    DEPTH(depth);
    fprintf(f, "%s %d/%s ", (cdt->dtree_op == DTREE_OP_EQ ? " =" : (cdt->dtree_op == DTREE_OP_LE ? "<=" : " >")), cdt->dtree_ival, cdt->dtree_sval);
    dtree_print(f, al, cdt, depth + 1);
    cdt = cdt->dtree_else;
  }
}
#endif

#undef DEPTH

