#include "eval_attribute.h"
#include "directions.h"
#include "wchar_utils.h"

#ifdef TRISH2
#  include <trish_dict.h>
#  include <patricia_search.h>
#else
#  include <patricia_node.h>
#  include <dictionary.h>
#endif
#include <string.h>

#include "pedantic_macros.h"
#include "mouchard.h"

#ifdef TRISH2
eval_attribute_t *eval_attribute(patricia_t *dict_list, int ctx_attr, attribute_t *attr, noun_phrase_t *np) {
  eval_attribute_t *ea = malloc(sizeof(eval_attribute_t));
  xmlNode **nl;
  int i;
  tdict_t **tdp;
  wchar_t *w;

  switch (attr->attr_type) {
  case ATTRIBUTE_TYPE_RE:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = regexec(((attr_re_t *)attr->attr_obj)->are_cre, np->np_form, 0 , NULL, 0) ? 0 : 1;
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_DICT:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_sval = NULL;
    w = char2wcs(((attr_dict_t *)attr->attr_obj)->adict_name);
    tdp = pmap_get(dict_list, w);
    if (*tdp != NULL) {
      psearch_t *search;
      switch (((attr_dict_t *)attr->attr_obj)->adict_op) {
      case ATTR_DICT_OP_EQUALS:
	search = psearch_new((*tdp)->map, PSEARCH_OPTION_FULL, NULL, 1);
	psearch_new_query(search, w);
	patricia_search(search);
	ea->ea_ival = (search->final != NULL);
	psearch_free(search);
	break;
      case ATTR_DICT_OP_CONTAINS:
	search = psearch_new((*tdp)->map, 0, NULL, 1);
	psearch_new_query(search, w);
	patricia_search(search);
	ea->ea_ival = (search->final != NULL);
	psearch_free(search);
	break;
      }
    }
    else {
      free(ea);
      ea = NULL;
    }
    free(w);
    break;
  case ATTRIBUTE_TYPE_LENGTH:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = strlen(np->np_form);
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_SIGNAL:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_sval = NULL;
    switch (((attr_signal_t *)attr->attr_obj)->asig_direction) {
    case DIRECTION_FOLLOWING:
      nl = np->np_following;
      break;
    case DIRECTION_PRECEDING:
      nl = np->np_preceding;
      break;
    }
    ea->ea_ival = 0;
    i = 0;
    while (nl[i] && (i < ((attr_signal_t *)attr->attr_obj)->asig_range) && (!ea->ea_ival)) {
      xmlChar *s;
      if (!strcmp(nl[i]->name, "ne")) {
	s = nl[i]->children->content;
      }
      else if (!strcmp(nl[i]->name, "word")) {
	xmlAttr *a = xmlHasProp(nl[i], "lemma");
	s = a->children->content;
      }
      if (!strcmp(s, ((attr_signal_t *)attr->attr_obj)->asig_word)) {
	ea->ea_ival = 1;
      }
      i++;
    }
    break;
  case ATTRIBUTE_TYPE_POS:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_STRING;
    ea->ea_ival = 0;
    switch (((attr_pos_t *)attr->attr_obj)->apos_direction) {
    case DIRECTION_FOLLOWING:
      nl = np->np_following;
      break;
    case DIRECTION_PRECEDING:
      nl = np->np_preceding;
      break;
    }
    if ((nl[((attr_pos_t *)attr->attr_obj)->apos_range] == NULL) || (nl[((attr_pos_t *)attr->attr_obj)->apos_range] == (void *)0x11 /* XXX ??? */))
      break;
    if (!strcmp(nl[((attr_pos_t *)attr->attr_obj)->apos_range]->name, "ne")) {
      ea->ea_sval = "N";
    }
    else if (!strcmp(nl[((attr_pos_t *)attr->attr_obj)->apos_range]->name, "word")) {
      xmlAttr *a;
      a = xmlHasProp(nl[((attr_pos_t *)attr->attr_obj)->apos_range], "category");
      ea->ea_sval = (char *)a->children->content;
    }
    break;
  case ATTRIBUTE_TYPE_CONTEXT:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = ctx_attr;
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_LABEL:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = 0;
    ea->ea_sval = NULL;
    break;
  }

  return ea;
}

eval_attribute_t *eval_attribute_name(patricia_t *dict_list, int ctx_attr, patricia_t *attr_list, const char *aname, noun_phrase_t *np) {
  wchar_t *w = char2wcs(aname);
  attribute_t *attr = pmap_get(attr_list, w);
  eval_attribute_t *ea = eval_attribute(dict_list, ctx_attr, attr, np);
  free(w);
  return ea;
}

#else

eval_attribute_t *eval_attribute(association_t *dict_list, int ctx_attr, attribute_t *attr, noun_phrase_t *np) {
  eval_attribute_t *ea = malloc(sizeof(eval_attribute_t));
  /*sp_bunch_t *spb;*/
  xmlNode **nl;
  int i;
  bunch_element_ref_t ref;

  switch (attr->attr_type) {
  case ATTRIBUTE_TYPE_RE:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = regexec(((attr_re_t *)attr->attr_obj)->are_cre, np->np_form, 0 , NULL, 0) ? 0 : 1;
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_DICT:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_sval = NULL;
    ref = association_get(dict_list, ((attr_dict_t *)attr->attr_obj)->adict_name);
    if (ref) {
      dictionary_t *dict = (*BUNCH_ELT(dictionary_t *, dict_list->values, ref));
      search_bunch_t *sb = patricia_quick_search(dict->dict_pat, np->np_form, PATRICIA_OPT_WHOLE, NULL /* XXX */);
      switch (((attr_dict_t *)attr->attr_obj)->adict_op) {
      case ATTR_DICT_OP_EQUALS:
	if (sb->first_used) {
	  ea->ea_ival = 1;
	  ea->ea_sval = (char *)lbl_ref_to_string(dict->dict_lbl, PNODE_LABEL(dict->dict_pat->pat_nodes, SEARCH_NODE(sb, sb->first_used)));
	}
	else {
	  ea->ea_ival = 0;
	}
	search_bunch_free(sb);
	break;
      case ATTR_DICT_OP_CONTAINS:
	if (sb->first_used) {
	  ea->ea_ival = 0;
	}
	else {
	  search_bunch_free(sb);
	  sb = search_bunch_new(np->np_form, NULL /* XXX */);
	  patricia_search((*BUNCH_ELT(dictionary_t *, dict_list->values, ref))->dict_pat, sb, 0, strlen(np->np_form) + 1);
	  if (sb->first_used) {
	    ea->ea_ival = 1;
	  }
	  else {
	    ea->ea_ival = 0;
	  }
	}
	search_bunch_free(sb);
	break;
      }
    }
    else {
      free(ea);
      ea = NULL;
    }
    break;
  case ATTRIBUTE_TYPE_LENGTH:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = strlen(np->np_form);
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_SIGNAL:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_sval = NULL;
    switch (((attr_signal_t *)attr->attr_obj)->asig_direction) {
    case DIRECTION_FOLLOWING:
      nl = np->np_following;
      break;
    case DIRECTION_PRECEDING:
      nl = np->np_preceding;
      break;
    }
    ea->ea_ival = 0;
    i = 0;
    while (nl[i] && (i < ((attr_signal_t *)attr->attr_obj)->asig_range) && (!ea->ea_ival)) {
      xmlChar *s;
      if (!strcmp(nl[i]->name, "ne")) {
	s = nl[i]->children->content;
      }
      else if (!strcmp(nl[i]->name, "word")) {
	xmlAttr *a = xmlHasProp(nl[i], "lemma");
	s = a->children->content;
      }
      if (!strcmp(s, ((attr_signal_t *)attr->attr_obj)->asig_word)) {
	ea->ea_ival = 1;
      }
      i++;
    }
    break;
  case ATTRIBUTE_TYPE_POS:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_STRING;
    ea->ea_ival = 0;
    switch (((attr_pos_t *)attr->attr_obj)->apos_direction) {
    case DIRECTION_FOLLOWING:
      nl = np->np_following;
      break;
    case DIRECTION_PRECEDING:
      nl = np->np_preceding;
      break;
    }
    if (!strcmp(nl[((attr_pos_t *)attr->attr_obj)->apos_range]->name, "ne")) {
      ea->ea_sval = "N";
    }
    else if (!strcmp(nl[((attr_pos_t *)attr->attr_obj)->apos_range]->name, "word")) {
      xmlAttr *a;
      a = xmlHasProp(nl[((attr_pos_t *)attr->attr_obj)->apos_range], "category");
      ea->ea_sval = (char *)a->children->content;
    }
    break;
  case ATTRIBUTE_TYPE_CONTEXT:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = ctx_attr;
    ea->ea_sval = NULL;
    break;
  case ATTRIBUTE_TYPE_LABEL:
    ea->ea_type = EVAL_ATTRIBUTE_TYPE_INT;
    ea->ea_ival = 0;
    ea->ea_sval = NULL;
    break;
  }

  return ea;
}

eval_attribute_t *eval_attribute_ref(association_t *dict_list, int ctx_attr, association_t *attr_list, bunch_element_ref_t aref, noun_phrase_t *np) {
  return eval_attribute(dict_list, ctx_attr, BUNCH_ELT(attribute_t, attr_list->values, aref), np);
}

eval_attribute_t *eval_attribute_name(association_t *dict_list, int ctx_attr, association_t *attr_list, const char *aname, noun_phrase_t *np) {
  return eval_attribute_ref(dict_list, ctx_attr, attr_list, association_get(attr_list, aname), np);
}
#endif
