#include "noun_phrase.h"
#include "directions.h"
#include <string.h>

#include "pedantic_macros.h"

#include "mouchard.h"

void np_get_context(noun_phrase_t *np, int dir);

noun_phrase_t *np_new(xmlNode *n, int frange, int prange) {
  noun_phrase_t *np;

  if (n->type != XML_ELEMENT_NODE) {
    return NULL;
  }

  if (!strcmp(n->name, "word")) {
    xmlAttr *attr = xmlHasProp(n, "category");
    if (attr == NULL) {
      return NULL;
    }
    if (attr->children->content[0] != 'N') {
      return NULL;
    }
  }
  else if (strcmp(n->name, "ne")) {
    return NULL;
  }

  np = calloc(1, sizeof(noun_phrase_t));

  np->np_length = 1;
  np->np_nodes = calloc(1, sizeof(xmlNode *));
  np->np_nodes[0] = n;

  np->np_form = calloc(sizeof(char), strlen(n->children->content) + 1);
  np->np_canonical = NULL;
  strcat(np->np_form, n->children->content);

  np->np_following_range = frange;
  np_get_context(np, DIRECTION_FOLLOWING);
  np->np_preceding_range = prange;
  np_get_context(np, DIRECTION_PRECEDING);

  return np;
}

void np_free(noun_phrase_t *np) {
  free(np->np_nodes);
  free(np->np_form);
  free(np->np_following);
  free(np->np_preceding);
  free(np);
}

int np_extend(noun_phrase_t *np, int maxl) {
  xmlNode *n;

  if (maxl == np->np_length)
    return 0;

  n = np->np_nodes[np->np_length - 1]->prev;
  if (n == NULL) {
    return 0;
  }

  if (n->type == XML_TEXT_NODE) {
    int i;
    for (i = 0; n->content[i]; i++) {
      switch (n->content[i]) {
      case ' ':
      case '\n':
      case '\r':
      case '\t':
	break;
      default:
	return 0;
      }
    }
    n = n->prev;
    if (n == NULL) {
      return 0;
    }
  }

  if (n->type == XML_ELEMENT_NODE) {
    xmlAttr *attr = xmlHasProp(n, "category");
    if (
	(!strcmp(n->name, "ne"))
	|| (
	    (!strcmp(n->name, "word"))
	    && (attr != NULL)
	    && (
		(attr->children->content[0] == 'N')
		|| (attr->children->content[0] == 'J')
		)
	    )
	)
      {
	int l = strlen(n->children->content);
	char *form = calloc(sizeof(char), l + strlen(np->np_form) + 2);
	strcat(form, n->children->content);
	form[l] = ' ';
	form[l + 1] = '\0';
	strcat(form, np->np_form);
	free(np->np_form);
	np->np_form = form;

	np->np_length++;
	np->np_nodes = realloc(np->np_nodes, sizeof(xmlNode *) * np->np_length);
	np->np_nodes[np->np_length - 1] = n;

	free(np->np_preceding);
	np_get_context(np, DIRECTION_PRECEDING);

	return 1;
      }
  }

  return 0;
}

void np_get_context(noun_phrase_t *np, int dir) {
  int i = 0;
  int range;
  xmlNode *n;
  xmlNode **nlist;

  switch (dir) {
  case DIRECTION_FOLLOWING:
    range = np->np_following_range;
    nlist = calloc(range, sizeof(xmlNode *));
    np->np_following = nlist;
    n = np->np_nodes[0]->next;
    break;
  case DIRECTION_PRECEDING:
    range = np->np_preceding_range;
    nlist = calloc(range, sizeof(xmlNode *));
    np->np_preceding = nlist;
    n = np->np_nodes[np->np_length - 1]->prev;
    break;
  }

  while ((n != NULL) && (i < range)) {
    if ((n->type == XML_ELEMENT_NODE) && ((!strcmp(n->name,"word")) || (!strcmp(n->name,"ne")))) {
      nlist[i] = n;
      i++;
    }
    switch (dir) {
    case DIRECTION_FOLLOWING:
      n = n->next;
      break;
    case DIRECTION_PRECEDING:
      n = n->prev;
      break;
    }
  }

  while (i < range) {
    nlist[i] = NULL;
    i++;
  }
}
